#ifndef AISL_CLIENT_H_164FE6B2_E5D4_4968_B50F_823E30E8F777
#define AISL_CLIENT_H_164FE6B2_E5D4_4968_B50F_823E30E8F777

#include <arpa/inet.h>

#include <aisl/client.h>

#define AISL_CLIENT(x) ((aisl_client_t) x)

#ifndef AISL_WITHOUT_SSL
/**
 * @brief Constructor for #aisl_client_t instance.
 * @param server an #aisl_server_t instance pointer.
 * @param fd a client socket descriptor.
 * @param addr a pointer to client's address structure.
 * @param ssl_ctx a pointer to SSL context or NULL if encryption is disabled
 */
aisl_client_t
aisl_client_new( aisl_server_t        server,
                 int                  fd,
                 struct sockaddr_in * addr,
                 SSL_CTX            * ssl_ctx);
#else

/**
 * @brief Constructor for #aisl_client_t instance.
 * @param server an #aisl_server_t instance pointer.
 * @param fd a client socket descriptor.
 * @param addr a pointer to client's address structure.
 * @param ssl_ctx a pointer to SSL context or NULL if encryption is disabled
 */
aisl_client_t
aisl_client_new( aisl_server_t        server,
                 int                  fd,
                 struct sockaddr_in * addr );
#endif


/**
 * @brief Destructor for #aisl_client_t instance.
 * @param client an #aisl_client_t instance pointer.
 */
void
aisl_client_free(aisl_client_t client);


/**
 * @brief Does all HTTP client routines.
 * Reads and parses requests, writes responses.
 * @param client an #aisl_client_t instance pointer.
 * @return #aisl_status_t code.
 */
aisl_status_t
aisl_client_touch(aisl_client_t client);


/**
 * @brief Controls if communication time is not timed out.
 * @param client an #aisl_client_t instance pointer.
 * @param timeout an allowed client silence time in seconds.
 * @return true if client communication is timed out, otherwise false.
 */
bool
aisl_client_is_timed_out(aisl_client_t client, uint32_t timeout);


/**
 * @brief Gets socket descriptor associated with #aisl_client_t instance.
 * @param client an #aisl_client_t instance pointer.
 * @return a client socket descriptor.
 */
int
aisl_client_get_socket(aisl_client_t client);

#endif /* !AISL_CLIENT_H */
